<?php

namespace Drupal\varnish_file_purge\Hook;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\file\FileInterface;
use Drupal\varnish_file_purge\Services\VarnishFilePurger;
use Symfony\Component\DependencyInjection\Attribute\Autowire;

/**
 * File hooks implementations.
 */
final class FileHooks {

  /**
   * Constructs a new EntityHooks service.
   *
   * @param \Drupal\varnish_file_purge\Services\VarnishFilePurger $varnishFilePurger
   *   The varnish file purger.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   The file system.
   */
  public function __construct(
    #[Autowire(service: 'varnish_file_purge.purger')] protected VarnishFilePurger $varnishFilePurger,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected FileSystemInterface $fileSystem,
  ) {}

  /**
   * Implements hook_ENTITY_TYPE_insert().
   *
   * Purge file urls when a file is created.
   */
  #[Hook('file_insert')]
  public function fileInsert(FileInterface $file): void {
    if (!$this->varnishFilePurger->isConfigured()) {
      return;
    }

    $this->varnishFilePurger->purge($file);
  }

  /**
   * Implements hook_ENTITY_TYPE_update().
   *
   * Purge file urls when a file is updated.
   */
  #[Hook('file_update')]
  public function fileUpdate(FileInterface $file): void {
    if (!$this->varnishFilePurger->isConfigured()) {
      return;
    }

    /** @var \Drupal\file\FileInterface $original_file */
    $original_file = $file->original;

    if (!$original_file instanceof FileInterface) {
      return;
    }

    if ($this->isFileDifferent($file, $original_file)) {
      $this->varnishFilePurger->purge($file);
    }
  }

  /**
   * Track changes in the updated file by comparing it with the original.
   *
   * @param \Drupal\file\FileInterface $file
   *   The updated file.
   * @param \Drupal\file\FileInterface $original_file
   *   The original file.
   *
   * @return bool
   *   Whether the file is different from the original file.
   */
  protected function isFileDifferent(FileInterface $file, FileInterface $original_file): bool {
    // Compare file sizes and URIs as a quick check (optional).
    if ($original_file->getSize() !== $file->getSize()) {
      return TRUE;
    }

    // Compare file URIs to check if the file was moved or renamed.
    if ($file->getFileUri() != $original_file->getFileUri()) {
      return TRUE;
    }

    // The file URI is the same, but with the file_replace module,
    // content might still have changed.
    // Therefore, compare file contents using hashes.
    $original_file_path = $this->fileSystem->realpath($original_file->getFileUri());
    $updated_file_path = $this->fileSystem->realpath($file->getFileUri());

    if (empty($original_file_path) || empty($updated_file_path)) {
      return TRUE;
    }

    // Generate hashes for both files.
    $original_file_hash = hash_file('sha256', $original_file_path);
    $updated_file_hash = hash_file('sha256', $updated_file_path);

    // Compare the hashes. If they differ, the file content has changed.
    if ($original_file_hash !== $updated_file_hash) {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * Implements hook_ENTITY_TYPE_delete().
   *
   * Purge file urls when a file is deleted.
   */
  #[Hook('file_delete')]
  public function fileDelete(FileInterface $file): void {
    if (!$this->varnishFilePurger->isConfigured()) {
      return;
    }

    $this->varnishFilePurger->purge($file);
  }

}
